@@ -12,6 +12,11 @@ import System.Random
 
 import Control.Monad
   ( liftM
+  , ap
+  )
+
+import Control.Applicative
+  ( Applicative(..)
   )
 
 import Control.Monad.Reader()
@@ -31,6 +36,10 @@ instance Functor Gen where
   fmap f (MkGen h) =
     MkGen (\r n -> f (h r n))
 
+instance Applicative Gen where
+  pure  = return
+  (<*>) = ap
+
 instance Monad Gen where
   return x =
     MkGen (\_ _ -> x)
